@@ -19,12 +19,12 @@
 
 /**
  * Biloba - Q1 2005
-	* Game by Guillaume Demougeot <dmgt@wanadoo.fr>
-	* Code by Colin Leroy <colin@colino.net>
-	*
-	* This file contains the board handling code (drawing
-	* and placing pawns).
-	*/
+ * Game by Guillaume Demougeot <dmgt@wanadoo.fr>
+ * Code by Colin Leroy <colin@colino.net>
+ *
+ * This file contains the board handling code (drawing
+ * and placing pawns).
+ */
 
 #include <SDL.h>
 #include <SDL_image.h>
@@ -41,6 +41,21 @@
 static SDL_Surface *border = NULL;
 static SDL_Surface *player = NULL;
 
+static void board_show_help(void)
+{
+	static SDL_Surface *help = NULL;
+#ifndef MAEMO
+	if (help == NULL)
+		help = biloba_load_image("commands.png");
+#else
+	if (help == NULL)
+		help = biloba_load_image("commands-maemo.png");
+#endif
+	assert(help != NULL);
+		
+	put_image(help, HELP_X, HELP_Y);
+}
+
 /**
  * Build the playing board from scratch, and display it on screen
  */
@@ -55,7 +70,7 @@ void board_build(void)
 			tile_draw(tile_get(x, y));
 		}
 	}
-	
+
 	for (nb_players = 0; nb_players < game_num_players(); nb_players++) {
 		Player *p = NULL;
 		for (num = 0; num < pawn_get_max(); num++) {
@@ -80,7 +95,8 @@ void board_build(void)
 	}
 	put_image(border, X_OFFSET - 36, Y_OFFSET - 36);
 	put_image(player, 5, YS - 65);
-	
+
+	board_show_help();	
 	SDL_UpdateRect(screen, 0, 0, 0, 0);
 }
 
@@ -93,15 +109,6 @@ void board_destroy(void)
 	pawn_free_all();
 }
 
-/**
- * Refresh the board image (border)
- */
-static void board_refresh(void)
-{
-	assert(border != NULL);
-	put_image(border, X_OFFSET - 36, Y_OFFSET - 36);
-}
-
 /**
  * Set the player turn
  *
@@ -115,8 +122,6 @@ void board_set_player(Player *player, int display_your_turn)
 	char *turn;
 	int len;
 	
-	board_refresh();
-
 	if (display_your_turn)
 		len = strlen(get_msg(M_YOUR_TURN)) + strlen(player->name) + 1;
 	else
