@@ -367,7 +367,8 @@ Tile *computer_select_tile(Player *player, LList *allowed)
 		goto out;
 
 	while (game_suspended()) {
-		SDL_Delay(20);
+		if (delay_with_event_poll(20) < 0)
+			goto out;
 	}
 	if (game_ended())
 		goto out;
@@ -378,7 +379,7 @@ Tile *computer_select_tile(Player *player, LList *allowed)
 	
 out:
 	while (game_suspended()) {
-		SDL_Delay(20);
+		delay_with_event_poll(20);
 	}
 	if (game_ended())
 		goto no_push;
