@@ -126,6 +126,14 @@ void draw_message_with_update(const char *message, int x, int y, int width,
 	}
 
 	for (i = first_char; i < last_char; i ++) {
+		if (message[i] == '\n') {
+			cur_x = x;
+			y += LINEHEIGHT;
+			if (!cursor)
+				last_char += i;
+			continue;
+		}
+
 		dst_rect->x = cur_x;
 		dst_rect->y = y + LINEYOFFSET;
 		
