@@ -20,8 +20,12 @@
 #ifndef __FONT_H__
 #define __FONT_H__
 
-void draw_message_with_update(const char *message, int x, int y, int cursor, int update);
-void draw_message(const char *message, int x, int y, int cursor);
+#define CHARWIDTH 10
+#define LINEYOFFSET (+3)
+#define LINEHEIGHT 18
+
+void draw_message_with_update(const char *message, int x, int y, int width, int cursor, int update);
+void draw_message(const char *message, int x, int y, int width, int cursor);
 void clear_text(int length, int x, int y);
 
 #endif
