@@ -25,28 +25,51 @@
 #include <SDL_mutex.h>
 
 typedef struct _Game Game;
+typedef struct _Move Move;
+typedef struct _Message Message;
+
+struct _Move
+{
+    	int32_t player;
+        int32_t x;
+        int32_t y;
+};
+
+struct _Message
+{
+    	int32_t player;
+        char message[256];
+};
 
 struct _Game
 {
 	char *name;
-	int id;
-	int num_players;
-	int num_remote;
+	int32_t id;
+	int32_t num_players;
+	int32_t num_remote;
 	char *player_name[4];
-	int player_type[4];
-	int first_avail_spot;
-	int started;
-	int cur_player, cur_x, cur_y;
-	int move_id;
-	int waiting_acks;
-	int quit_players;
-	int killed;
-	int last_joined;
+	int32_t player_type[4];
+	int32_t first_avail_spot;
+	int32_t started;
+	Move *moves;
+	int32_t move_id;
+	int32_t players_move_id[4];
+	int32_t quit_players;
+	int32_t killed;
+	int32_t last_joined;
+
+	Message *messages;
+	int32_t message_id;
+	int32_t players_msg_id[4];
 
 	SDL_cond *cond_wait;
 	SDL_cond *cond_move;
 	SDL_cond *cond_quit;
 };
 
-void game_free(Game *);
+void game_push_move(Game *game, int player, int x, int y);
+void game_push_message(Game *game, int player, const char *msg);
+void free_game(Game *game);
+Game *init_game(void);
+
 #endif
