@@ -30,6 +30,8 @@ struct _LList {
 LList *llist_append(LList *list, void *data);
 LList *llist_prepend(LList *list, void *data);
 LList *llist_remove(LList *list, void *data);
+LList *llist_copy(LList *list);
+LList *llist_reverse(LList *list);
 LList *llist_find(LList *list, void *data);
 void   llist_free(LList *list);
 int    llist_length(LList *list);
