@@ -24,10 +24,7 @@
 #include "pawn.h"
 #include "player.h"
 
-int compute_eats(PawnColor color);
 void play_game(void);
-void stop_game(void);
-void ask_replacements(Player *player);
 int game_ended(void);
 void end_game(void);
 int can_move_on_center(void);
