@@ -30,6 +30,9 @@
 #include <string.h>
 #include <SDL.h>
 #include <SDL_image.h>
+#include <time.h>
+#include <unistd.h>
+
 #include "utils.h"
 #include "board.h"
 #include "logic.h"
@@ -45,15 +48,21 @@ static void run_tests()
 
 int main(int argc, char *argv[])
 {
-	if (!strncmp(argv[0], DIR_SEP, strlen(DIR_SEP))) {
+	const char *execname = argv[0];
+	if (execname[0] >= 'A' && execname[0] <= 'Z' && execname[1] == ':') {
+		/* Windows: Skip C: */
+		execname += 2;
+	}
+	if (!strncmp(execname, DIR_SEP, strlen(DIR_SEP))) {
 		/* absolute path */
 		char *last_dirsep = NULL;
-		progpath = strdup(argv[0]);
+		progpath = strdup(execname);
 		last_dirsep = progpath;
 		while (strstr(last_dirsep + 1, DIR_SEP) != NULL)
 			last_dirsep = strstr(last_dirsep + 1, DIR_SEP);
-		
 		*last_dirsep = '\0';
+		if (chdir(progpath) < 0)
+			printf("Couldn't chdir to %s\n", progpath);
 	}
 	
 	SDL_Init(SDL_INIT_EVERYTHING|SDL_INIT_NOPARACHUTE);
@@ -86,7 +95,7 @@ int main(int argc, char *argv[])
 	while (!should_quit()) {
 		game_init(FALSE);
 
-		if (get_options() < 0)
+		if (get_options(argc > 1 ? argv[1]:NULL) < 0)
 			break;
 
 		if (should_quit())
@@ -99,6 +108,9 @@ int main(int argc, char *argv[])
 		play_game();
 		
 		board_destroy();
+
+		if (argc > 1)
+			break;
 	}
 
 	sound_stop();
