@@ -35,6 +35,8 @@ typedef enum {
 	M_YOUR_TURN,
 	M_OFFICIAL_SERVER,
 	M_LOCAL_SERVER,
+	M_SAVED_TO,
+	M_SAVE_FAIL,
 	M_NUM_MSGS
 } MsgId;
 
