@@ -30,6 +30,7 @@
 #include <stdio.h>
 #include <errno.h>
 #include <sys/types.h>
+#include <time.h>
 #ifndef __MINGW32__
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -52,6 +53,7 @@
 #include "game.h"
 #include "msg.h"
 #include "server.h"
+#include "options.h"
 
 #undef NET_DEBUG
 
@@ -79,8 +81,9 @@ void net_stop(void)
 #endif
 
 static int game_running = FALSE;
-static int mysock = INVSOCK;
-static int mygameid = -1;
+static int main_sock = INVSOCK;
+static int32_t mygameid = -1;
+static int32_t mygameplayer = -1;
 
 #ifdef __MINGW32__
 void debug_err(int y, int err)
@@ -148,7 +151,7 @@ static int sock_connect(const char *hostname, unsigned short port)
 #endif
 		SDL_Delay(200);
 #endif
-		return -1;
+		return INVSOCK;
 	} else {
 #ifdef NET_DEBUG
 		draw_message("socket ok", 10, 10, -1, FALSE);
@@ -157,9 +160,9 @@ static int sock_connect(const char *hostname, unsigned short port)
 	}
 	
 	if (hostname == NULL)
-		return -1;
+		return INVSOCK;
 	
-	printf("connecting to %s\n", hostname);
+	printf("\nConnecting to %s...\n", hostname);
 	server = gethostbyname(hostname);
 	if (server == NULL) {
 #ifdef NET_DEBUG
@@ -170,7 +173,7 @@ static int sock_connect(const char *hostname, unsigned short port)
 #endif
 		SDL_Delay(200);
 #endif
-		return -1;
+		return INVSOCK;
 	} else {
 #ifdef NET_DEBUG
 		draw_message("gethostbyname ok", 10, 42, -1, FALSE);
@@ -184,14 +187,12 @@ static int sock_connect(const char *hostname, unsigned short port)
 		server->h_length);
 	serv_addr.sin_port = htons(port);
 
-	printf("connecting\n");
-
 	err = connect(sockfd, (struct sockaddr *)&serv_addr, 
 		      sizeof(serv_addr));
 #ifndef __MINGW32__	
 	if (err < 0) {
 #else
-	if (err == INVALID_SOCKET) {
+	if (err == SOCKET_ERROR) {
 #endif
 #ifdef NET_DEBUG
 		draw_message("connect error", 10, 74, -1, FALSE);
@@ -201,351 +202,640 @@ static int sock_connect(const char *hostname, unsigned short port)
 #endif
 		SDL_Delay(200);
 #endif
-		printf("connect error: %d\n", err);
-		return -1;
+		printf("Connection error: %d\n", err);
+		return INVSOCK;
 	} else {
 #ifdef NET_DEBUG
 		draw_message("connect ok", 10, 74, -1, FALSE);
 		SDL_Delay(200);
 #endif
 	}
-	printf("connected: fd %d\n", sockfd);
+	printf("Connected.\n");
 	return (sockfd);
 }
 
-#define CHECK_MSG(str) {				\
-	read_msg(mysock, buf, 255, &len);		\
-	buf[254] = '\0';				\
-	if (strncmp(buf, str, strlen(str))) {		\
-		printf("Unexpected reply \"%s\"",	\
-			buf);				\
+#define CHECK_MSG(s, str) {				\
+	r = read_str(s, buf, 255);			\
+	if (r <= 0 || strncmp(buf, str, strlen(str))) {	\
+		printf("Unexpected reply \"%s\", "	\
+			"expected \"%s\"\n",		\
+			buf, str);			\
 		goto err_protocol;			\
 	}						\
 }
 
-#define DUMP(buf) {\
-	int k=0;\
-	for(k=0; k < 255; k++)\
-		printf("%c(%d) ",buf[k],buf[k]);\
-	printf("\n");\
-}
-
 int net_init_game(const char *gamename, int numplayers,
-		  InputSystemMethod p0method, const char *p0name,
-		  InputSystemMethod p1method, const char *p1name,
-		  InputSystemMethod p2method, const char *p2name,
-		  InputSystemMethod p3method, const char *p3name)
+		  InputSystemMethod methods[], char *names[])
 {
 	char buf[255];
-	unsigned char len = 0;
-	int i = 0;
-	unsigned int gameid;
+	int i = 0, n, r;
 	assert(game_running == FALSE);
 	
 	SDL_FillRect(screen, NULL, 0x00000000);
 	SDL_UpdateRect(screen, 0, 0, 0, 0);
 
-	mysock = sock_connect(net_get_server(), NET_PORT);
-	printf("INIT: sock %d\n", mysock);
-#ifndef __MINGW32__
-	if (mysock < 0)
-		goto err_socket;
-#else
-	if (mysock == INVALID_SOCKET)
-		goto err_socket;
-#endif
+	main_sock = sock_connect(net_get_server(), NET_PORT);
+	if (main_sock == INVSOCK)
+		return -1;
+
+	printf("\n");
 
 	/* init connection */
-	send_msg(mysock, "NEWGAME", strlen("NEWGAME")+1);
-	printf("INIT> NEWGAME\n");
-	CHECK_MSG("OK");
-	printf("INIT< OK\n");
+	r = send_str(main_sock, "NEWGAME");
+	if (r < 0) {
+		printf("Error sending NEWGAME: %s(%d)\n", net_err_str(r), r);
+		goto err_protocol;
+	}
+	printf("> Sending game creation command\n");
+	CHECK_MSG(main_sock, "OK");
 
 	/* send name */
 	assert(strlen(gamename) < 255);
-	send_msg(mysock, gamename, (unsigned char)strlen(gamename)+1);
-	printf("INIT> %s\n", gamename);
-
-	/* get id */
-	read_msg(mysock, &gameid, 4, &len);
-	if (len > 4) {
-		printf("unexpected len %d on id\n", len);
+	r = send_str(main_sock, gamename);
+	if (r < 0) {
+		printf("Error sending game name: %s(%d)\n", net_err_str(r), r);
 		goto err_protocol;
 	}
-	mygameid = ntohl(gameid);
-	printf("INIT< %d (game id)\n", mygameid);
+	printf("> Sending game name '%s'\n", gamename);
+
+	/* get id */
+	r = read_int(main_sock, &mygameid);
+	if (r < 0) {
+		printf("Error reading game ID: %s(%d)\n", net_err_str(r), r);
+		goto err_protocol;
+	}
+	printf("< Received assigned game ID %d\n", mygameid);
 
-#ifdef NET_DEBUG
-	printf("mygameid: %d\n", mygameid);
-#endif
 	/* send nump, methods and names */
 	buf[i] = (unsigned char)numplayers; i++;
-			
-	buf[i] = (unsigned char)p0method; i++;
-	strcpy(buf+i, p0name); i+=strlen(p0name)+1;
 	
-	buf[i] = (unsigned char)p1method; i++;
-	strcpy(buf+i, p1name); i+=strlen(p1name)+1;
-
-	if(numplayers > 2) {
-		buf[i] = (unsigned char)p2method; i++;
-		strcpy(buf+i, p2name); i+=strlen(p2name)+1;
+	for (n = 0; n < numplayers; n++) {
+		buf[i] = (unsigned char)methods[n];
+		i++;
+		strcpy(buf+i, names[n]);
+		i+=strlen(names[n])+1;
+		printf("> Sending player %d (%s) name '%s'\n", 
+			n, input_system_names(methods[n]),
+			names[n]);
 	}
-	if(numplayers > 3) {
-		buf[i] = (unsigned char)p3method; i++;
-		strcpy(buf+i, p3name); i+=strlen(p3name)+1;
+	r = send_msg(main_sock, buf, i);
+	if (r < 0) {
+		printf("Error sending player details: %s(%d)\n", net_err_str(r), r);
+		goto err_protocol;
 	}
-	send_msg(mysock, buf, i);
-	printf("INIT> PLAYER NAMES AND TYPES\n");
-	
-	CHECK_MSG("READY");
-	printf("INIT< READY\n");
+	CHECK_MSG(main_sock, "READY");
+	printf("< Received ack, now waiting for other players\n");
 
+	mygameplayer = 0;
 	return mygameid;
 	
 err_protocol:
-	printf("closing.\n");
-	close(mysock);
-err_socket:
-	mysock = INVSOCK;
+	printf("Closing connection.\n");
+	net_close();
 	mygameid = -1;
 	return -1;
 }
 
+static int talk_sock = INVSOCK;
+static SDL_Thread *net_talk_thread = NULL;
+static int talk_thread_run = 1;
+
+static int msg_available_for_send = FALSE;
+static char waiting_send_msg[256];
+static int msg_available_for_recv = FALSE;
+static char waiting_recv_msg[256];
+static SDL_mutex *msg_mutex = NULL;
+
+static int typing_msg = FALSE;
+static char msg[MAX_MSG_LEN] = "";
+static int msg_len = 0;
+
+int net_is_typing_msg(void)
+{
+	return typing_msg;
+}
+
+static void net_talk_push_send_msg(const char *buf)
+{
+	if (net_talk_thread == NULL)
+		return;
+
+	SDL_LockMutex(msg_mutex);
+
+	printf("> Sending message '%s'\n", buf);
+	strncpy(waiting_send_msg, buf, sizeof(waiting_send_msg));
+	waiting_send_msg[sizeof(waiting_send_msg) - 1] = '\0';
+
+	msg_available_for_send = TRUE;
+	
+	SDL_UnlockMutex(msg_mutex);	
+}
+
+void net_set_typing_msg(int typing)
+{
+	if (!options_using_net())
+		return;
+
+	typing_msg = typing;
+	if (!typing_msg && msg_len > 0) {
+		/* Send pending message */
+		net_talk_push_send_msg(msg);
+	}
+	printf("  %s typing message...\n", typing_msg ? "Started":"Stopped");
+	/* Reset buffer */
+	msg_len = 0;
+	msg[msg_len] = '\0';
+
+	if (typing) {
+		char talk_str[256];
+		clear_text(-1, G_TALK_X, G_TALK_Y);
+		snprintf(talk_str, sizeof(talk_str), "%s: ",
+			 options_get_player_name(mygameplayer));
+		draw_message(talk_str, G_TALK_X, G_TALK_Y,
+			     MAX_MSG_LEN * CHARWIDTH, TRUE);
+	} else {
+		clear_text(-1, G_TALK_X, G_TALK_Y);
+	}
+}
+
+const char *net_get_pending_msg(void)
+{
+	if (!typing_msg)
+		return NULL;
+	return msg;
+}
+
+void net_type_msg(unsigned char c)
+{
+	char talk_str[MAX_MSG_LEN + 1];
+
+	if (c == SDLK_ESCAPE) {
+		msg_len = 0;
+		net_set_typing_msg(FALSE);
+		return;
+	} else if (c == SDLK_BACKSPACE) {
+		c='\0';
+	} else if (c < 0x20 || c == 0x7f)
+		return;
+
+	assert(typing_msg == TRUE);
+
+	if (msg_len >= MAX_MSG_LEN 
+		- strlen(options_get_player_name(mygameplayer)) 
+		- strlen(": ")
+		- 1)
+		return;
+
+	if (c != '\0') {
+		msg[msg_len] = c;
+		msg_len++;
+		msg[msg_len] = '\0';
+	} else if (msg_len > 0) {
+		msg_len--;
+		msg[msg_len] = '\0';
+	}
+
+
+	snprintf(talk_str, sizeof(talk_str), "%s: %s",
+		 options_get_player_name(mygameplayer), msg);
+
+	clear_text(-1, G_TALK_X, G_TALK_Y);
+	draw_message(talk_str, G_TALK_X, G_TALK_Y, MAX_MSG_LEN * CHARWIDTH, TRUE);
+}
+
+static void net_talk_push_recv_msg(const char *player, const char *msg)
+{
+	if (player == NULL || msg == NULL)
+
+	SDL_LockMutex(msg_mutex);
+
+	if (player != NULL && msg != NULL)
+		snprintf(waiting_recv_msg, sizeof(waiting_recv_msg), "%s: %s", player, msg);
+	else
+		waiting_recv_msg[0] = '\0';
+
+	waiting_recv_msg[sizeof(waiting_recv_msg) - 1] = '\0';
+
+	msg_available_for_recv = TRUE;
+
+	SDL_UnlockMutex(msg_mutex);
+}
+
+void net_pull_recv_msg(void)
+{
+	char *msg = NULL;
+
+	SDL_LockMutex(msg_mutex);
+
+	if (msg_available_for_recv) {
+		msg_available_for_recv = FALSE;
+		msg = strdup(waiting_recv_msg);
+	}
+	SDL_UnlockMutex(msg_mutex);
+
+	if (msg) {
+		printf("> Pulled message '%s'\n", msg);
+		clear_text(-1, G_TALK_X, G_TALK_Y);
+		draw_message(msg, G_TALK_X, G_TALK_Y, MAX_MSG_LEN * CHARWIDTH, FALSE);
+		free(msg);
+	}
+}
+
+int net_talk_thread_func(void *data)
+{
+	char buf[256];
+	int r;
+	time_t last_displayed_message = time(NULL);
+	int should_clear = FALSE;
+
+	if (talk_sock != INVSOCK) {
+		sock_close(talk_sock);
+		talk_sock = INVSOCK;
+	}
+	
+	talk_sock = sock_connect(net_get_server(), NET_PORT);
+	
+	if (send_str(talk_sock, "TALK_MODE") < 0 ||
+	    send_int(talk_sock, mygameid) < 0 ||
+	    send_int(talk_sock, mygameplayer) < 0)
+		goto err_protocol;
+
+	CHECK_MSG(talk_sock, "OK");
+
+	while (talk_thread_run) {
+		Game *game;
+		if (talk_sock == INVSOCK)
+			break;
+
+		SDL_LockMutex(msg_mutex);
+		if (msg_available_for_send) {
+			printf("< Message available\n");
+			msg_available_for_send = FALSE;
+			if (send_str(talk_sock, waiting_send_msg) < 0) {
+				SDL_UnlockMutex(msg_mutex);
+				printf("can't send message\n");
+				break;
+			}
+			printf("> Sent message '%s'\n", waiting_send_msg);
+		}
+		SDL_UnlockMutex(msg_mutex);
+		if (sock_select(talk_sock, 500) >= 0) {
+			int32_t player;
+			char buf[256];
+			
+			if (read_int(talk_sock, &player) < 0 ||
+			    read_str(talk_sock, buf, 255) < 0) {
+				break;
+			}
+			printf("< Got message from player %d: %s\n", player, buf);
+
+			options_game_lock();
+			game = options_get_game();
+			if (!game) {
+				options_game_unlock();
+				printf("can't get game\n");
+				break;
+			}
+			game_push_message(game, player, buf);
+			options_game_unlock();
+		}
+
+		options_game_lock();
+		game = options_get_game();
+		if (!game) {
+			options_game_unlock();
+			printf("can't get game\n");
+			break;
+		}
+		if (game->message_id > game->players_msg_id[mygameplayer]
+                    && !net_is_typing_msg()) {
+			/* Display first available message. */
+			int msg_id = game->players_msg_id[mygameplayer] + 1;
+			const Message *msg = &game->messages[msg_id];
+			const char *player_name = game->player_name[msg->player];
+
+			if (msg->player != mygameplayer) {
+				last_displayed_message = time(NULL);
+				should_clear = TRUE;
+				net_talk_push_recv_msg(player_name, msg->message);
+			}
+
+			game->players_msg_id[mygameplayer]++;
+		} else if (game->message_id == game->players_msg_id[mygameplayer]
+			   && !net_is_typing_msg() && should_clear
+			   && time(NULL) - last_displayed_message > 60) {
+			net_talk_push_recv_msg(NULL, NULL);
+			should_clear = FALSE;
+		}
+		options_game_unlock();
+	}
+	
+err_protocol:
+	sock_close(talk_sock);
+	talk_sock = INVSOCK;
+	return 0;
+}
+
+int net_open_talk_socket(void)
+{
+	if (net_talk_thread) {
+		int tmp;
+		talk_thread_run = 0;
+		SDL_WaitThread(net_talk_thread, &tmp);
+	}
+
+	if (msg_mutex == NULL)
+		msg_mutex = SDL_CreateMutex();
+
+	talk_thread_run = 1;
+	net_talk_thread = SDL_CreateThread(net_talk_thread_func, NULL);
+	
+	return 0;
+}
+
 int net_wait_ready(int num_players, char **player_names)
 {
-	int waiting = 0, prev_waiting = 0;
-	unsigned char len = 0;
+	int missing_players = 0, prev_missing_players = 0;
 	char buf[255], player[255];
 	int nplayer, i;
 	
-	if (mysock == INVSOCK)
+	if (main_sock == INVSOCK)
 		return -1;
 
-#ifndef __MINGW32__
-	assert(mysock > 0);
-#else
-	assert(mysock != INVALID_SOCKET);
-#endif
 	do {
 		if (event_poll() < 0)
 			goto err_out;
 
-		if (send_msg(mysock, "READY?", 7) < 0)
+		if (send_str(main_sock, "READY?") < 0)
 			goto err_out;
-		if (read_msg(mysock, &waiting, 4, &len) < 0)
+		if (read_int(main_sock, &missing_players) < 0)
 			goto err_out;
-		waiting = ntohl(waiting);
 		sprintf(buf, get_msg(M_WAITING),
-				waiting, waiting > 1 ? "s":"");
+				missing_players, missing_players > 1 ? "s":"");
 		SDL_FillRect(screen, NULL, 0x00000000);
 		SDL_UpdateRect(screen, 0, 0, 0, 0);
 		draw_message(buf, 150, 284, -1, FALSE);
 		
-		if (read_msg(mysock, &nplayer, 4, &len) < 0)
+		if (read_int(main_sock, &nplayer) < 0)
 			goto err_out;
-		nplayer = ntohl(nplayer);
 
-		if (read_msg(mysock, &player, 255, &len) < 0)
+		if (read_str(main_sock, player, 255) < 0)
 			goto err_out;
-		player[254] = '\0';
-		if (prev_waiting > waiting) {
+
+		if (prev_missing_players > missing_players) {
+			Game *game;
 			sprintf(buf, get_msg(M_JOINED),
 					player);
 			draw_message(buf, 150, 284 + 40, -1, FALSE);
-			printf("%d: %s\n", nplayer, player);
+			printf("< Received a new player join: %d '%s'\n",
+				nplayer, player);
+
+			/* Fixme the two player names overlap. */
 			strcpy(player_names[nplayer], player);
+			options_game_lock();
+			game = options_get_game();
+			if (game) {
+				free(game->player_name[nplayer]);
+				game->player_name[nplayer] = strdup(player);
+			}
+			options_game_unlock();
 			if (delay_with_event_poll(1000) < 0)
 				goto err_out;
 		}
 		
-		prev_waiting = waiting;
+		prev_missing_players = missing_players;
 		
 		if (delay_with_event_poll(1000) < 0)
 			goto err_out;
-	} while (waiting > 0);
+	} while (missing_players > 0);
+
+	printf("\n");
 
 	for (i = 0; i < num_players; i++) {
-		if (read_msg(mysock, &player, 255, &len) < 0)
+		if (read_str(main_sock, player, 255) < 0)
 			return -1;
-		player[254] = '\0';
 		strcpy(player_names[i], player);
 	}
-	return 0;
+
+	return net_open_talk_socket();
 err_out:
-	sock_close(mysock);
-	mysock = INVSOCK;
+	net_close();
 	return -1;
 }
 
 LList *net_get_games(int numplayer)
 {
 	char buf[255];
-	unsigned char len = 0;
-	int i = 0, numgames = 0;
+	int i = 0, numgames = 0, r;
 	LList *games = NULL;
 	assert(game_running == FALSE);
-	
-#ifndef __MINGW32
-	if (mysock > 0) {
-#else
-	if (mysock != INVALID_SOCKET) {
-#endif
-		close(mysock);
-		mysock = INVSOCK;
-	}
 
-	mysock = sock_connect(net_get_server(), NET_PORT);
-	printf("LISTGAME: sock %d\n", mysock);
-#ifndef __MINGW32__
-	if (mysock < 0)
-		goto err_socket;
-#else
-	if (mysock == INVALID_SOCKET)
-		goto err_socket;
-#endif
+	/* We reconnect each time. Suboptimal. */
+	if (main_sock != INVSOCK)
+		net_close();
+
+	main_sock = sock_connect(net_get_server(), NET_PORT);
+	if (main_sock == INVSOCK)
+		return NULL;
+
+	printf("\n");
+
 	/* init connection */
-	send_msg(mysock, "LISTGAME", strlen("LISTGAME")+1);
-	printf("LISTGAME> LISTGAME\n");
-	CHECK_MSG("NUMP");
-	printf("LISTGAME< NUMP\n");
+	r = send_str(main_sock, "LISTGAME");
+	if (r < 0) {
+		printf("Sending list games command failed: %s(%d)\n",
+			net_err_str(r), r);
+		goto err_protocol;
+	}
+	printf("> Asking for games list\n");
+	CHECK_MSG(main_sock, "NUMP");
 	
 	/* send num players */
-	i = htonl(numplayer);
-	send_msg(mysock, &i, 4);
-	printf("LISTGAME> %d\n", numplayer);
+	r = send_int(main_sock, numplayer);
+	if (r < 0) {
+		printf("Sending number of players failed: %s(%d)\n",
+			net_err_str(r), r);
+		goto err_protocol;
+	}
+	printf("> Asking for games with %d players\n", numplayer);
 
 	/* read number of games */
-	read_msg(mysock, &numgames, 4, &len);
-	numgames = ntohl(numgames);
-	printf("LISTGAME< %d (numgames)\n", numgames);
+	r = read_int(main_sock, &numgames);
+	if (r < 0) {
+		printf("Getting number of games failed: %s(%d)\n",
+			net_err_str(r), r);
+		goto err_protocol;
+	}
+	printf("< Will receive %d available games\n", numgames);
 
 	for (i = 0; i < numgames; i++) {
-		int id = 0;
 		Game *game = malloc(sizeof(Game));
+
 		memset(game, 0, sizeof(Game));
-		read_msg(mysock, &id, 4, &len);
-		game->id = ntohl(id);
-		printf("LISTGAME< %d (id)\n", game->id);
-		read_msg(mysock, buf, 255, &len);
-		buf[254] = '\0';
-		game->name = strdup(buf);
-		printf("LISTGAME< %s (name)\n", game->name);
+
 		game->num_players = numplayer;
+
+		r = read_int(main_sock, &game->id);
+		if (r < 0) {
+			printf("Getting game ID failed: %s(%d)\n",
+				net_err_str(r), r);
+			free(game);
+			goto err_protocol;
+		}
+
+		r = read_str(main_sock, buf, 255);
+		if (r < 0) {
+			printf("Getting game name failed: %s(%d)\n",
+				net_err_str(r), r);
+			free(game);
+			goto err_protocol;
+		}
+		game->name = strdup(buf);
+		printf("< Received game ID %d, '%s'\n", game->id, game->name);
 		games = llist_append(games, game);
 	}
 	
 	return games;
 
 err_protocol:
-	close(mysock);
-err_socket:
-	mysock = INVSOCK;
+	net_close();
 	return NULL;
 }
 
 int net_get_info(Game *game)
 {
-	int gameid = htonl(game->id);
 	char buf[255];
-	unsigned char len;
-	int tmp, i;
+	int i, r;
 
-	printf("PRELJOIN: sock %d\n", mysock);
-
-	if (mysock == INVSOCK)
+	if (main_sock == INVSOCK)
 		return -1;
-#ifndef __MINGW32__
-	assert(mysock > 0);
-#else
-	assert(mysock != INVALID_SOCKET);
-#endif
-	send_msg(mysock, "PRELJOIN", strlen("PRELJOIN") + 1);
-	printf("PRELJOIN> PRELJOIN\n");
-	send_msg(mysock, &gameid, 4);
-	printf("PRELJOIN> %d (id)\n", game->id);
+
+	printf("\n");
+
+	r = send_str(main_sock, "PRELJOIN");
+	if (r < 0) {
+		printf("Sending preliminary join command failed: %s(%d)\n",
+			net_err_str(r), r);
+		goto err_protocol;
+	}
+
+	r = send_int(main_sock, game->id);
+	if (r < 0) {
+		printf("Sending game ID failed: %s(%d)\n", net_err_str(r), r);
+		goto err_protocol;
+	}
+	printf("> Asking for game ID %d details\n", game->id);
 
 	memset(buf, 0, 255);
-	read_msg(mysock, buf, 255, &len);
-	buf[254] = '\0';
-	printf("PRELJOIN< %s (answer)\n", buf);
+	r = read_str(main_sock, buf, 255);
+	if (r < 0) {
+		printf("Getting anwser failed: %s(%d)\n", net_err_str(r), r);
+		goto err_protocol;
+	}
 
 	if (!strncmp(buf, "ERROR1", 6)) {
-		close(mysock);
-		mysock = INVSOCK;
-		return -1;
+		printf("Error getting information (game %d doesn't exist)\n",
+			game->id);
+		goto err_protocol;
 	}
 
 	for (i = 0; i < game->num_players; i++) {
-		read_msg(mysock, buf, 255, &len);
-		buf[254] = '\0';
-		printf("PRELJOIN< %s (player name)\n", buf);
+		r = read_str(main_sock, buf, 255);
+		if (r < 0) {
+			printf("Failed getting player %d's name: %s(%d)\n", i,
+				net_err_str(r), r);
+			goto err_protocol;
+		}
+		printf("< Received player %d's name '%s')\n", i, buf);
 		game->player_name[i] = strdup(buf);
 		game->player_type[i] = INPUT_NETWORK;
 	}
 	
-	read_msg(mysock, &tmp, 4, &len);
-	tmp = ntohl(tmp);
-	printf("PRELJOIN< %d (first avail spot)\n", tmp);
-	game->first_avail_spot = tmp;
+	r = read_int(main_sock, &game->first_avail_spot);
+	if (r < 0) {
+		printf("Getting first available spot failed: %s(%d)\n",
+			net_err_str(r), r);
+		goto err_protocol;
+	}
+	printf("< Received first available spot %d)\n", game->first_avail_spot);
 	
 	return 0;
+
+err_protocol:
+	net_close();
+	return -1;
 }
 
 int net_join(int id, int nump, const char *my_player_name)
 {
 	char buf[255];
-	unsigned char len;
-	int mid = htonl(nump);
+	int r;
 
-	if (mysock == INVSOCK)
+	if (main_sock == INVSOCK)
 		return -1;
 
-	printf("JOIN: sock %d\n", mysock);
-#ifndef __MINGW32__
-	assert(mysock > 0);
-#else
-	assert(mysock != INVALID_SOCKET);
-#endif
-	
-	send_msg(mysock, "JOIN", strlen("JOIN") + 1);
-	printf("JOIN> JOIN\n");
-	send_msg(mysock, &mid, 4);
-	printf("JOIN> %d (num player)\n", nump);
-	read_msg(mysock, buf, 255, &len);
-	buf[254] = '\0';
-	printf("JOIN< %s (answer)\n", buf);
+	printf("\n");
+
+	r = send_str(main_sock, "JOIN");
+	if (r < 0) {
+		printf("Sending join command failed: %s(%d)\n", net_err_str(r), r);
+		goto err_protocol;
+	}
+	printf("> Sending join command\n");
+	r = send_int(main_sock, nump);
+	if (r < 0) {
+		printf("Sending player number failed: %s(%d)\n", net_err_str(r), r);
+		goto err_protocol;
+	}
+	printf("> Sending my player number %d\n", nump);
+
+	r = read_str(main_sock, buf, 255);
+	if (r < 0) {
+		printf("Reading join reply failed: %s(%d)\n", net_err_str(r), r);
+		goto err_protocol;
+	}
+
 	if (strncmp(buf, "OK", 2)) {
-		close(mysock);
-		mysock = INVSOCK;
-		return -1;
+		printf("Join refused.\n");
+		goto err_protocol;
 	}
 	
-	send_msg(mysock, my_player_name, strlen(my_player_name) + 1);
-	printf("JOIN> %s (my name)\n", my_player_name);
-	
+	r = send_str(main_sock, my_player_name);
+	if (r < 0) {
+		printf("Sending player name failed: %s(%d)\n", net_err_str(r), r);
+		goto err_protocol;
+	}
+	printf("> Sending my player name '%s'\n", my_player_name);
+
+	printf("\n");
+
+	mygameid = id;
+	mygameplayer = nump;
+
 	return 0;
+
+err_protocol:
+	net_close();
+	return -1;
 }
 
 void net_send_event(int player, int x, int y)
 {
-	char cmd = 'p';
-	int tmp;
-	
-#ifndef __MINGW32__
-	if (mysock < 0)
+	int r;
+
+	if(main_sock == INVSOCK)
 		return;
-#else
-	if(mysock == INVALID_SOCKET)
+
+	if ((r = send_char(main_sock, 'p')) < 0 ||
+	    (r = send_int(main_sock, player)) < 0 ||
+	    (r = send_int(main_sock, x)) < 0 ||
+	    (r = send_int(main_sock, y)) < 0) {
+		printf("> Error sending event: %s (%d)\n", net_err_str(r), r);
+		net_close();
 		return;
-#endif
-	send_msg(mysock, &cmd, 1);
-	tmp = htonl(player);
-	send_msg(mysock, &tmp, 4);
-	tmp = htonl(x);
-	send_msg(mysock, &tmp, 4);
-	tmp = htonl(y);
-	send_msg(mysock, &tmp, 4);
+	}
+
+	printf("> Sending player %d event x %d, y %d\n", player, x, y);
 }
 
 static SDL_Thread *net_event_thread = NULL;
@@ -561,35 +851,37 @@ typedef struct {
 static int net_event_thread_func(void *data)
 {
 	NetData *vals = data;
-	char cmd = 'r';
-	int tmp, tmp_x, tmp_y;
-	unsigned char len;
-	
-#ifndef __MINGW32__
-	if (mysock < 0) {
-#else
-	if(mysock == INVALID_SOCKET) {
-#endif
+	int r;
+	int32_t x = -1, y = -1;
+
+	if(main_sock == INVSOCK) {
 		vals->x = -2;
 		vals->y = -2;
 		SDL_SemPost(net_event_thread_sem);
 		return -1;
 	}
-
-	send_msg(mysock, &cmd, 1);
-	tmp = htonl(vals->player);
-	if (send_msg(mysock, &tmp, 4) < 0 ||
-	    read_msg(mysock, &tmp_x, 4, &len) < 0 ||
-	    read_msg(mysock, &tmp_y, 4, &len) < 0) {
+again:
+	printf("< Asking for player event\n");
+	if ((r = send_char(main_sock, 'r')) < 0 ||
+	    (r = send_int(main_sock, vals->player)) < 0 ||
+	    (r = read_int(main_sock, &x)) < 0 ||
+	    (r = read_int(main_sock, &y)) < 0) {
 		SDL_LockMutex(net_event_thread_mutex);
 		vals->x = -2;
 		vals->y = -2;
+		printf("< Error getting player %d event: %s(%d)\n",
+			vals->player, net_err_str(r), r);
 		SDL_UnlockMutex(net_event_thread_mutex);
 		SDL_SemPost(net_event_thread_sem);
+		return -1;
 	} else {
+		if (x == -1 && y == -1)
+			goto again;
 		SDL_LockMutex(net_event_thread_mutex);
-		vals->x = ntohl(tmp_x);
-		vals->y = ntohl(tmp_y);
+		vals->x = x;
+		vals->y = y;
+		printf("> Got player %d event x %d, y %d\n", vals->player,
+			vals->x, vals->y);
 		SDL_UnlockMutex(net_event_thread_mutex);
 		SDL_SemPost(net_event_thread_sem);
 	}
@@ -603,8 +895,8 @@ void net_get_event(int player, int *x, int *y)
 	int tmp;
 
 	data.player = player;
-	data.x = -1;
-	data.y = -1;
+	data.x = *x = -1;
+	data.y = *y = -1;
 
 	net_event_thread_sem = SDL_CreateSemaphore(0);
 	net_event_thread_mutex = SDL_CreateMutex();
@@ -614,17 +906,18 @@ void net_get_event(int player, int *x, int *y)
 	while (data.x == -1 && data.y == -1) {
 		SDL_UnlockMutex(net_event_thread_mutex);
 		SDL_UpdateRect(screen, 0, 0, 0, 0);
-		SDL_SemWaitTimeout(net_event_thread_sem, 500);
+		SDL_SemWaitTimeout(net_event_thread_sem, 50);
 		if (event_poll() < 0) {
 			SDL_KillThread(net_event_thread);
 			net_event_thread = NULL;
 			data.x = -2;
 			data.y = -2;
-			sock_close(mysock);
+			net_close();
 			break;
 		}
 		SDL_LockMutex(net_event_thread_mutex);
 	}
+	SDL_UnlockMutex(net_event_thread_mutex);
 
 	if (net_event_thread)
 		SDL_WaitThread(net_event_thread, &tmp);
@@ -637,31 +930,27 @@ void net_get_event(int player, int *x, int *y)
 
 void net_end_game(int player)
 {
-	char cmd = 'q';
-	int tmp;
-#ifndef __MINGW32__
-	if (mysock < 0)
+	int r;
+
+	if(main_sock == INVSOCK)
 		return;
-#else
-	if(mysock == INVALID_SOCKET)
-		return;
-#endif	
-	send_msg(mysock, &cmd, 1);
-	tmp = htonl(player);
-	send_msg(mysock, &tmp, 4);
+
+	if ((r = send_char(main_sock, 'q')) < 0 ||
+	    (r = send_int(main_sock, player)) < 0) {
+		printf("> Error sending quit event: %s (%d)\n", net_err_str(r), r);
+		net_close();
+	}
+
+	printf("> Sending quit event for player %d\n", player);
 }
 
 void net_close(void)
 {
-#ifndef __MINGW32__
-	if (mysock < 0)
+	if(main_sock == INVSOCK)
 		return;
-#else
-	if(mysock == INVALID_SOCKET)
-		return;
-#endif	
-	sock_close(mysock);
-	mysock = INVSOCK;
+
+	sock_close(main_sock);
+	main_sock = INVSOCK;
 }
 
 static SDL_Thread *server_thread = NULL;
@@ -670,7 +959,7 @@ static int server_main_thread(void *data)
 {
 	server_set_select_interval(1);
 	server_set_do_broadcast(1, net_magic_number);
-	return server_start(NULL);
+	return server_start(NULL, NULL);
 }
 
 void net_set_server(const char *srv)
@@ -785,12 +1074,8 @@ static int scan_for_servers(void *data)
 					(struct sockaddr *)&addr,
 					&sock_len);
 		if (ret < 0) {
-#ifdef __MINGW32__
-			ret = WSAGetLastError();
-#else
-			ret = errno;
-#endif
-			printf("err: %s (%d)\n", strerror(ret), ret);
+			ret = get_net_error();
+			printf("error %s (%d)\n", net_err_str(ret), ret);
 		}
  
 		if (ret == BCAST_LEN && !strncmp(mybuf, "Biloba", 6)
