@@ -28,10 +28,7 @@
 void net_init(void);
 void net_stop(void);
 int net_init_game(const char *gamename, int numplayers,
-		  InputSystemMethod p0method, const char *p0name,
-		  InputSystemMethod p1method, const char *p1name,
-		  InputSystemMethod p2method, const char *p2name,
-		  InputSystemMethod p3method, const char *p3name);
+		  InputSystemMethod methods[], char *names[]);
 int net_wait_ready(int num_players, char **player_names);
 LList *net_get_games(int numplayer);
 int net_get_info(Game *game);
@@ -41,8 +38,14 @@ void net_get_event(int player, int *x, int *y);
 void net_end_game(int player);
 void net_close(void);
 void net_set_server(const char *srv);
-
 int net_get_servers(char ***distant_servers);
 void net_stop_getting_servers(void);
 
+void net_pull_recv_msg(void);
+
+int net_is_typing_msg(void);
+void net_set_typing_msg(int typing);
+const char *net_get_pending_msg(void);
+void net_type_msg(unsigned char c);
+
 #endif
