@@ -41,9 +41,20 @@
 
 int send_msg(int fd, const void* buf, unsigned char len);
 int read_msg(int fd, void *buf, unsigned char avail, unsigned char *len);
+
+int send_int(int fd, int32_t n);
+int read_int(int fd, int32_t *n);
+
+int send_char(int fd, char c);
+int read_char(int fd, char *c);
+
+int send_str(int fd, const char *str);
+int read_str(int fd, char *str, int max_len);
+
 int sock_select(int sock, int timeout_msec);
 void sock_close(int sock);
 void get_bcast_msg(char *msg, size_t len, int magic_number);
 const char *get_hostname(struct sockaddr_in *addr);
-
+int get_net_error(void);
+const char *net_err_str(int err);
 #endif
