@@ -33,7 +33,11 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <time.h>
+#include <errno.h>
+#include <limits.h>
+#include <libgen.h>
 
+#include "options.h"
 #include "utils.h"
 #include "arrow.h"
 #include "font.h"
@@ -43,77 +47,65 @@
 #include "help.h"
 #include "msg.h"
 #include "widget.h"
+#include "replay.h"
 
-#define BILOBA_MIN_X (300)
-#define BILOBA_MIN_Y (20)
+#define BILOBA_MIN_X 		 (300)
+#define NUM_PLAYERS_MIN_X 	 (100)
+#define PLAYERS_MIN_X 		 (50)
+#define GAMENAME_MIN_X 		 (50)
+#define SERVER_MIN_X 		 (50)
+#define CHOOSE_SERVER_MIN_X 	 (410)
+#define CREATEGAME_MIN_X 	 (50)
+#define JOINGAME_MIN_X 		 (300)
+#define START_MIN_X 		 (550)
+#define HELP_MIN_X 		 (720)
+#define LANG_MIN_X 		 (688)
+#define PLAYERPAWN_MIN_X(i) 	 (100)
+#define PLAYERNAME_MIN_X(i) 	 (140)
+#define PLAYERTYPE_MIN_X(i) 	 (340)
+#define NETWORK_GAMES_LIST_MIN_X (550)
+#define GAMELIST_MIN_X(i) 	 (550)
+#define SAVE_LAST_MIN_X 	 (550)
+#define SAVED_MIN_X		 (100)
 
-#define NUM_PLAYERS_MIN_X (100)
-#define PLAYERS_MIN_X (50)
-#define PLAYERS_MIN_Y (100)
-
-#define GAMENAME_MIN_X (50)
 #ifndef MAEMO
-#define GAMENAME_MIN_Y (400)
+#define BILOBA_MIN_Y		 (20)
+#define PLAYERS_MIN_Y		 (100)
+#define GAMENAME_MIN_Y		 (400)
+#define SERVER_MIN_Y		 (350)
+#define CHOOSE_SERVER_MIN_Y	 (350)
+#define CREATEGAME_MIN_Y	 (500)
+#define JOINGAME_MIN_Y		 (500)
+#define START_MIN_Y		 (500)
+#define HELP_MIN_Y		 (40)
+#define LANG_MIN_Y		 (40)
+#define PLAYERPAWN_MIN_Y(i)	 (200 + (40*(i-2)))
+#define PLAYERNAME_MIN_Y(i)	 (200 + (40*(i-2)))
+#define PLAYERTYPE_MIN_Y(i)	 (200 + (40*(i-2)))
+#define NETWORK_GAMES_LIST_MIN_Y (100)
+#define GAMELIST_MIN_Y(i)	 (151 + (40*(i)))
+#define SAVE_LAST_MIN_Y		 (400)
+#define SAVED_MIN_Y		 (275)
 #else
-#define GAMENAME_MIN_Y (372)
+#define BILOBA_MIN_Y		 (10)
+#define PLAYERS_MIN_Y		 (70)
+#define GAMENAME_MIN_Y		 (367)
+#define SERVER_MIN_Y		 (320)
+#define CHOOSE_SERVER_MIN_Y	 (320)
+#define CREATEGAME_MIN_Y	 (420)
+#define JOINGAME_MIN_Y		 (420)
+#define START_MIN_Y		 (420)
+#define HELP_MIN_Y		 (30)
+#define LANG_MIN_Y		 (30)
+#define PLAYERPAWN_MIN_Y(i)	 (170 + (40*(i-2)))
+#define PLAYERNAME_MIN_Y(i)	 (170 + (40*(i-2)))
+#define PLAYERTYPE_MIN_Y(i)	 (170 + (40*(i-2)))
+#define NETWORK_GAMES_LIST_MIN_Y (70)
+#define GAMELIST_MIN_Y(i)	 (121 + (40*(i)))
+#define SAVE_LAST_MIN_Y		 (367)
+#define SAVED_MIN_Y		 (215)
 #endif
 
-#define SERVER_MIN_X (50)
-#ifndef MAEMO
-#define SERVER_MIN_Y (350)
-#else
-#define SERVER_MIN_Y (325)
-#endif
-
-#define CHOOSE_SERVER_MIN_X (410)
-#ifndef MAEMO
-#define CHOOSE_SERVER_MIN_Y (350)
-#else
-#define CHOOSE_SERVER_MIN_Y (325)
-#endif
-
-#define CREATEGAME_MIN_X (50)
-#ifndef MAEMO
-#define CREATEGAME_MIN_Y (500)
-#else
-#define CREATEGAME_MIN_Y (420)
-#endif
-
-#define JOINGAME_MIN_X (300)
-#ifndef MAEMO
-#define JOINGAME_MIN_Y (500)
-#else
-#define JOINGAME_MIN_Y (420)
-#endif
-
-#define START_MIN_X (550)
-#ifndef MAEMO
-#define START_MIN_Y (500)
-#else
-#define START_MIN_Y (420)
-#endif
-
-#define HELP_MIN_X (720)
-#define HELP_MIN_Y (40)
-
-#define LANG_MIN_X (688)
-#define LANG_MIN_Y (40)
-
-#define PLAYERPAWN_MIN_X(i) (100)
-#define PLAYERPAWN_MIN_Y(i) (200 + (40*(i-2)))
-
-#define PLAYERNAME_MIN_X(i) (140)
-#define PLAYERNAME_MIN_Y(i) (200 + (40*(i-2)))
-
-#define PLAYERTYPE_MIN_X(i) (340)
-#define PLAYERTYPE_MIN_Y(i) (200 + (40*(i-2)))
-
-#define network_games_list_MIN_X (500)
-#define network_games_list_MIN_Y (100)
-
-#define GAMELIST_MIN_X(i) (500)
-#define GAMELIST_MIN_Y(i) (151 + (40*(i)))
-
 /* GUI widgets */
 static Widget *numplayers = NULL;
 static Widget *numplayers_icon = NULL;
@@ -132,6 +124,8 @@ static Widget *join_game = NULL;
 static Widget *start = NULL;
 static Widget *help = NULL;
 static Widget *lang = NULL;
+static Widget *save_last = NULL;
+static Widget *saved = NULL;
 
 /* Options states */
 static int num_players = 2;
@@ -158,6 +152,23 @@ static const char *get_input_type_img(InputSystemMethod input)
 	return "";
 }
 
+static Game *cur_game = NULL;
+static SDL_mutex *cur_game_mutex = NULL;
+
+Game *options_get_game(void)
+{
+	return cur_game;
+}
+
+void options_game_lock(void)
+{
+	SDL_LockMutex(cur_game_mutex);
+}
+
+void options_game_unlock(void)
+{
+	SDL_UnlockMutex(cur_game_mutex);
+}
 /**
  * Get the player number's from a widget
  *
@@ -278,7 +289,7 @@ static void update_games_list(void)
 	LList *cur;
 	int i;
 
-	llist_for_each(network_games_list, game_free);
+	llist_for_each(network_games_list, free_game);
 	llist_free(network_games_list);
 	network_games_list = NULL;
 
@@ -472,6 +483,59 @@ static void do_start(Widget *widget, void *data)
 	start_clicked = 1;
 }
 
+/**
+ * Signal the GUI to save the last played game.
+ *
+ * @param[in]	widget		The Play widget
+ * @param[in]	data		Unused callback data
+ */
+static void do_save_last(Widget *widget, void *data)
+{
+	char str[SAVED_LEN];
+
+	const char *dir = get_desktop_folder();
+	char basename_dir[PATH_MAX];
+	char filename[PATH_MAX];
+	char path[PATH_MAX];
+
+	time_t now = time(NULL);
+	struct tm *tm;
+
+	if (!folder_exists(dir))
+		dir = get_home_folder();
+
+	if (!folder_exists(dir)) {
+		snprintf(str, SAVED_LEN, "%s %s", get_msg(M_SAVE_FAIL),
+			dir);
+		return;
+	}
+
+	tm = localtime(&now);
+
+	snprintf(filename, PATH_MAX, "Biloba-%04d-%02d-%02d-%02d%02d.%02d.blb",
+			tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
+			tm->tm_hour, tm->tm_min, tm->tm_sec);
+	filename[PATH_MAX - 1] = '\0';
+
+	snprintf(path, PATH_MAX, "%s%s%s", dir, DIR_SEP, filename);
+
+	path[PATH_MAX - 1] = '\0';
+
+	if (cur_game) {
+		strncpy(basename_dir, dir, sizeof(basename_dir));
+
+		if (replay_dump_game(cur_game, path) == 0)
+			snprintf(str, SAVED_LEN, "%s: %s%s\n%s", get_msg(M_SAVED_TO),
+				basename(basename_dir), DIR_SEP, filename);
+		else
+			snprintf(str, SAVED_LEN, "%s:\n%s", get_msg(M_SAVE_FAIL),
+				path);
+
+		text_widget_set_text(saved, str);
+		widget_show(saved);
+	}
+}
+
 /**
  * Signal the GUI after clicking the Help button.
  *
@@ -491,15 +555,26 @@ static void do_help(Widget *widget, void *data)
  */
 static void set_lang(Widget *widget, void *data)
 {
-	if (!strcmp(langpath,"en"))
+	if (!strcmp(langpath,"en")) {
 		langpath="fr";
-	else if (!strcmp(langpath,"fr"))
+		widget_set_image(lang, "fr.png");
+	}
+	else if (!strcmp(langpath,"fr")) {
 		langpath="es";
-	else if (!strcmp(langpath,"es"))
+		widget_set_image(lang, "es.png");
+	}
+	else if (!strcmp(langpath,"es")) {
 		langpath="en";
+		widget_set_image(lang, "en.png");
+	}
 	gui_reload_images();
 }
 
+static void hide_saved(Widget *widget, void *data)
+{
+	widget_hide(saved);
+}
+
 static int server_chooser_get_servers(Widget *widget, char ***strings)
 {
 	char **servers;
@@ -585,7 +660,7 @@ static void build_gui(void)
 					NULL, NULL);
 	text_widget_init(game_name, get_msg(M_NEWGAME), GAME_NAME_LEN, 111, 0, NULL, NULL);
 	
-	network_games =	 widget_create("network-games.png", network_games_list_MIN_X, network_games_list_MIN_Y,
+	network_games =	 widget_create("network-games.png", NETWORK_GAMES_LIST_MIN_X, NETWORK_GAMES_LIST_MIN_Y,
 					NULL, NULL);
 	widget_hide(network_games);
 	widget_hide(server);
@@ -605,10 +680,19 @@ static void build_gui(void)
 					do_join_game, NULL);
 	start =	 	 widget_create("start.png", START_MIN_X, START_MIN_Y,
 					do_start, NULL);
+	save_last = 	 widget_create("save_last.png", SAVE_LAST_MIN_X, SAVE_LAST_MIN_Y,
+					do_save_last, NULL);
+
 	help =		 widget_create("help.png", HELP_MIN_X, HELP_MIN_Y,
 					do_help, NULL);
-	lang =	 	 widget_create("fr.png", LANG_MIN_X, LANG_MIN_Y,
-					set_lang, "fr");
+	lang =	 	 widget_create("en.png", LANG_MIN_X, LANG_MIN_Y,
+					set_lang, NULL);
+
+	saved = 	 widget_create("saved.png", SAVED_MIN_X, SAVED_MIN_Y, hide_saved, NULL);
+	text_widget_init(saved, "", SAVED_LEN, 10, 0, NULL, NULL);
+	text_widget_set_editable(saved, FALSE);
+
+	widget_hide(saved);
 
 	/* FIXME: Setting server_chooser is done last just because it's the
 	 * easiest way to make sure it takes focus over the rest */
@@ -642,16 +726,21 @@ static void reset_net_widgets(void)
 	widget_hide(server_chooser);
 }
 
+static int use_net = FALSE;
+int options_using_net(void)
+{
+	return use_net;
+}
 /**
  * Main GUI loop to get the options
  *
  * @return 0 if the game can start, -1 if the caller should quit.
  */
-int get_options(void)
+int get_options(const char *replay_file)
 {
-	int use_net = FALSE;
 	static int initialized = FALSE;
 	int net_game_number = -1;
+	int i;
 
 	if (!initialized)
 	{
@@ -659,7 +748,34 @@ int get_options(void)
 		initialized = TRUE;
 	}
 
+	widget_hide(saved);
+
+	if (cur_game && cur_game->move_id > 1)
+		widget_show(save_last);
+	else
+		widget_hide(save_last);
+
 	reset_net_widgets();
+	use_net = FALSE;
+
+	if (replay_file) {
+		struct stat st;
+		int i;
+
+		if (stat(replay_file, &st) != 0 || !S_ISREG(st.st_mode))
+			return -1;
+	
+		if (replay_game_setup(replay_file) < 0)
+			return -1;
+
+		set_num_players(replay_get_num_players());
+		for (i = 0; i < replay_get_num_players(); i++) {
+			text_widget_set_text(player_name[i],
+				replay_get_player(i));
+			player_types[i] = INPUT_REPLAY;
+		}
+		return 0;
+	}
 
 choose_again:			
 	start_clicked = 0;
@@ -673,11 +789,8 @@ choose_again:
 
 	if (network_create_enabled) {
 		use_net = TRUE;
-		net_game_number = net_init_game(text_widget_get_text(game_name), num_players,
-				player_types[0], player_names[0],
-				player_types[1], player_names[1],
-				player_types[2], player_names[2],
-				player_types[3], player_names[3]);
+		net_game_number = net_init_game(text_widget_get_text(game_name),
+                                num_players, player_types, player_names);
 	} else if (network_join_enabled && selected_net_game_id > 0) {
 		use_net = TRUE;
 		net_game_number = selected_net_game_id;
@@ -685,15 +798,35 @@ choose_again:
 			 text_widget_get_text(player_name[selected_net_game_player_slot]));
 	}
 
+	if (cur_game_mutex == NULL)
+		cur_game_mutex = SDL_CreateMutex();
+
+	options_game_lock();
+
+	free_game(cur_game);
+	cur_game = init_game();
+	cur_game->num_players = num_players;
+
+	for (i = 0; i < num_players; i++)
+		cur_game->player_name[i] = strdup(options_get_player_name(i));
+
+	options_game_unlock();
+
 	if (use_net && (net_game_number < 0 || net_wait_ready(num_players, player_names) < 0))
 	{
 		SDL_FillRect(screen, NULL, 0x00000000);
 		SDL_UpdateRect(screen, 0, 0, 0, 0);
 		draw_message(get_msg(M_ENOCONN), 200, 284, -1, FALSE);
 
-		llist_for_each(network_games_list, game_free);
+		llist_for_each(network_games_list, free_game);
 		llist_free(network_games_list);
 		network_games_list = NULL;
+
+		options_game_lock();
+		free_game(cur_game);
+		cur_game = NULL;
+		options_game_unlock();
+
 		if (delay_with_event_poll(2000) < 0)
 			use_net = FALSE;
 		else
@@ -702,12 +835,13 @@ choose_again:
 	if (!use_net)
 		net_close();
 
-	llist_for_each(network_games_list, game_free);
+	llist_for_each(network_games_list, free_game);
 	llist_free(network_games_list);
 	network_games_list = NULL;
 
 	SDL_FillRect(screen, NULL, 0x00000000);
 	SDL_UpdateRect(screen, 0, 0, 0, 0);
+
 	return 0;
 }
 
