@@ -20,8 +20,16 @@
 #ifndef __OPTIONS_H__
 #define __OPTIONS_H__
 
-int get_options(void);
+#include "game.h"
+
+int get_options(const char *replay);
 const char *options_get_player_name(int i);
 int options_get_player_type(int i);
+int options_using_net(void);
+Game *options_get_game(void);
+void options_game_lock(void);
+void options_game_unlock(void);
+
+void replay_get_event(int color, int *x, int *y);
 
 #endif
