@@ -28,9 +28,23 @@ typedef enum {
 	INPUT_LOCAL,
 	INPUT_NETWORK,
 	INPUT_AI,
+	INPUT_REPLAY,
 	NUM_INPUT_SYSTEMS
 } InputSystemMethod;
 
+static inline char *input_system_names(InputSystemMethod i) {
+	char *name[3] = {
+			"LOCAL",
+			"NETWORK",
+			"AI" };
+	if (i < 0 || i >= NUM_INPUT_SYSTEMS)
+		return "WRONG TYPE";
+
+	return name[i];
+};
+
+
+        
 typedef struct _Player {
 	PawnColor color;
 	InputSystemMethod method;
