@@ -40,7 +40,8 @@ int main(int argc, char *argv[])
         WSAStartup(MAKEWORD(2,0),&wsaData);
 #endif	
 
-	err = server_start(argc > 1 ? argv[1] : NULL);
+	err = server_start(argc > 1 ? argv[1] : NULL,
+			   argc > 2 ? argv[2] : NULL);
 
 #ifdef __MINGW32__
 	WSACleanup();
