@@ -49,24 +49,17 @@
 #include <SDL_thread.h>
 #include <SDL_mutex.h>
 
+#include "server.h"
+#include "utils.h"
 #include "netops.h"
 #include "llist.h"
 #include "game.h"
 #include "player.h"
 #include "pawn.h"
+#include "replay.h"
 
 static FILE *logfp = NULL;
-
-static char *input_system_names(InputSystemMethod i) {
-	char *name[3] = {
-			"LOCAL",
-			"NETWORK",
-			"AI" };
-	if (i < 0 || i >= NUM_INPUT_SYSTEMS)
-		return "WRONG TYPE";
-
-	return name[i];
-};
+static const char *dump_dir = NULL;
 
 static unsigned int cur_id = 0;
 SDL_mutex *gamelist_mutex = NULL;
@@ -83,6 +76,7 @@ typedef enum {
 	CONN_SETUP_NEW,
 	CONN_SETUP_LISTGAME,
 	CONN_SETUP_PRELJOIN,
+	CONN_SETUP_TALK_MODE,
 	CONN_SETUP_JOIN,
 	CONN_WAIT_PLAYERS,
 	CONN_START_PLAY,
@@ -90,6 +84,9 @@ typedef enum {
 	CONN_PLAY_SEND_MOVE,
 	CONN_PLAY_SEND_QUIT,
 	CONN_PLAY_RECEIVE,
+	CONN_WAIT_MESSAGE,
+	CONN_SEND_MESSAGE,
+	CONN_RECV_MESSAGE,
 	CONN_KILL,
 	CONN_EXIT,
 	NUM_CONN_STATES
@@ -104,9 +101,9 @@ typedef struct _ConnData {
 	Game *game;
 	int previously_reserved;
 	int player_num;
-	int move_id;
 	int am_initiator;
 	int last_sent_join;
+	int talk_mode;
 } ConnData;
 
 static ConnData *conn_data_init(int fd, const char *addr)
@@ -126,9 +123,9 @@ static ConnData *conn_data_init(int fd, const char *addr)
 	cd->game = NULL;
 	cd->previously_reserved = -1;
 	cd->player_num = -1;
-	cd->move_id = 0;
 	cd->am_initiator = 0;
 	cd->last_sent_join = -1;
+	cd->talk_mode = 0;
 
 	return cd;
 }
@@ -142,12 +139,16 @@ static char *state_to_str(ConnState state)
 	case CONN_SETUP_LISTGAME:	return "CONN_SETUP_LISTGAME";
 	case CONN_SETUP_PRELJOIN:	return "CONN_SETUP_PRELJOIN";
 	case CONN_SETUP_JOIN:		return "CONN_SETUP_JOIN";
+	case CONN_SETUP_TALK_MODE:	return "CONN_SETUP_TALK_MODE";
 	case CONN_WAIT_PLAYERS:		return "CONN_WAIT_PLAYERS";
 	case CONN_START_PLAY:		return "CONN_START_PLAY";
 	case CONN_PLAY:			return "CONN_PLAY";
 	case CONN_PLAY_SEND_MOVE:	return "CONN_PLAY_SEND_MOVE";
 	case CONN_PLAY_SEND_QUIT:	return "CONN_PLAY_SEND_QUIT";
 	case CONN_PLAY_RECEIVE:		return "CONN_PLAY_RECEIVE";
+	case CONN_WAIT_MESSAGE:		return "CONN_WAIT_MESSAGE";
+	case CONN_SEND_MESSAGE:		return "CONN_SEND_MESSAGE";
+	case CONN_RECV_MESSAGE:		return "CONN_RECV_MESSAGE";
 	case CONN_KILL:			return "CONN_KILL";
 	case CONN_EXIT:			return "CONN_EXIT";
 	default:			
@@ -169,7 +170,7 @@ static void do_log(ConnData *cd, const char *format, ...)
 #ifndef __MINGW32__
 	localtime_r(&t, &tm);
 
-        fprintf(logfp, "%02d/%02d/%4d %02d:%02d:%02d.%04d [%s(%d)] - ", 
+	fprintf(logfp, "%02d/%02d/%4d %02d:%02d:%02d.%04d [%s(%d)] - ", 
 		    tm.tm_mday,
 		    tm.tm_mon +1,
 		    tm.tm_year + 1900,
@@ -180,34 +181,42 @@ static void do_log(ConnData *cd, const char *format, ...)
 		    cd ? cd->addr : "Server ",
 		    cd ? cd->fd : 0);
 #else
-        fprintf(logfp, "%s [%s(%d)] - ", 
+	fprintf(logfp, "%s [%s(%d)] - ", 
 		    asctime(localtime(&t)),
 		    cd ? cd->addr : "Server ",
 		    cd ? cd->fd : 0);
 #endif
 
-        va_start(args, format);
-        vfprintf(logfp, format, args);
-        va_end(args);
+	va_start(args, format);
+	vfprintf(logfp, format, args);
+	va_end(args);
 	
 	fflush(logfp);
 }
 
-static void free_game(Game *game)
+static char *sanitize(char *str)
 {
-	int i = 0;
-	
-	if (!game)
-		return;
+	while (strstr(str, DIR_SEP))
+		*strstr(str, DIR_SEP) = '_';
 
-	free(game->name);
-	for (i = 0; i < 4; i++)
-		free(game->player_name[i]);
-	
-	SDL_DestroyCond(game->cond_wait);
-	SDL_DestroyCond(game->cond_move);
-	SDL_DestroyCond(game->cond_quit);
-	free(game);
+	return str;
+}
+
+static void dump_game(Game *game)
+{
+	char dump_file[512];
+
+	if (snprintf(dump_file, sizeof(dump_file), "%s%s%d-%s.blb",
+			dump_dir, DIR_SEP, game->id,
+			sanitize(game->name)) >= sizeof(dump_file)) {
+		do_log(NULL, "Can't dump game %d (%s): File name too long\n",
+			game->id, game->name);
+		return;
+	}
+
+	if (replay_dump_game(game, dump_file) < 0)
+		do_log(NULL, "Can't dump game %d (%s) to %s: %s\n",
+			game->id, game->name, dump_file, strerror(errno));
 }
 
 /* must be called with gamelist_mutex held */
@@ -283,33 +292,25 @@ static void set_timeouts(ConnData *cd, int seconds)
 static int server_setup_game(ConnData *cd, Game **new_game)
 {
 	char *p_name;
-	int gameid = 1;
 	char buf[256];
-	unsigned char len = 0, i, p;
+	unsigned char len, i, p;
 	int numplayers = 0;
 
-	*new_game = malloc(sizeof(struct _Game));
-	if (*new_game == NULL)
-		return -ENOMEM;
+	*new_game = init_game();
 
-	memset(*new_game, 0, sizeof(struct _Game));
-
-	if (send_msg(cd->fd, "OK", 3) < 0)
+	if (send_str(cd->fd, "OK") < 0)
 		goto error;
 
 	/* read game name */
-	if (read_msg(cd->fd, buf, 255, &len) < 0) 
+	if (read_str(cd->fd, buf, 255) < 0) 
 		goto error;
 
-	buf[254] = '\0';
-
 	do_log(cd, " > game name '%s'\n", buf);
 
 	(*new_game)->name = strdup(buf);
 
 	cur_id++;
-	gameid = htonl(cur_id);
-	if (send_msg(cd->fd, &gameid, 4) < 0)
+	if (send_int(cd->fd, cur_id) < 0)
 		goto error;
 
 	do_log(cd, " < game ID %d\n", cur_id);
@@ -322,14 +323,12 @@ static int server_setup_game(ConnData *cd, Game **new_game)
 	if (read_msg(cd->fd, buf, 255, &len) < 0)
 		goto error;
 
-	buf[254] = '\0';
-
 	i = 0;
 	numplayers = (int) buf[i]; i++;
 
 	do_log(cd, " > %d players\n", numplayers);
 	if (numplayers < 1 || numplayers > 4) {
-		send_msg(cd->fd, "ERROR", 6);
+		send_str(cd->fd, "ERROR");
 		do_log(cd, "   wrong number of players\n");
 		goto error;
 	}
@@ -339,12 +338,11 @@ static int server_setup_game(ConnData *cd, Game **new_game)
 
 
 	for (p = 0; p < numplayers; p++) {
-
 		(*new_game)->player_type[p] = (InputSystemMethod)buf[i];
 		
 		if ((*new_game)->player_type[p] < 0 ||
 		    (*new_game)->player_type[p] >= NUM_INPUT_SYSTEMS) {
-			send_msg(cd->fd, "ERROR", 6);
+			send_str(cd->fd, "ERROR");
 			do_log(cd, "   wrong player type %d for player %d\n",
 				(*new_game)->player_type[p], p);
 			goto error;
@@ -379,11 +377,11 @@ static int server_setup_game(ConnData *cd, Game **new_game)
 	if ((*new_game)->first_avail_spot == -1) {
 
 		do_log(cd, "   ERROR: could not get a spot for another player\n");
-		send_msg(cd->fd, "ERROR", 6);
+		send_str(cd->fd, "ERROR");
 		goto error;
 	}
 
-	if (send_msg(cd->fd, "READY",6) < 0)
+	if (send_str(cd->fd, "READY") < 0)
 		goto error;
 
 	do_log(cd, "   < game %s (%d) fully set up\n", (*new_game)->name,
@@ -406,34 +404,26 @@ error:
 static int server_get_requested_num_players(ConnData *cd)
 {
 	int nump;
-	unsigned char len;
 
-	if (send_msg(cd->fd, "NUMP", 5) < 0) {
+	if (send_str(cd->fd, "NUMP") < 0)
 		return -1;
-	}
 
-	if (read_msg(cd->fd, &nump, 4, &len) < 0) {
+	if (read_int(cd->fd, &nump) < 0)
 		return -1;
-	}
 
-	if (len > 4) {
-		return -1;
-	}
-	return ntohl(nump);
+	return nump;
 }
 
 static int server_send_games(ConnData *cd, int nump)
 {
 	LList *avail = NULL, *cur;
-	int num, err;
+	int err;
 
 	SDL_LockMutex (gamelist_mutex);
 
 	avail = get_games(nump, 6);
 
-	num = htonl(llist_length(avail));
-
-	if (send_msg(cd->fd, &num, 4) < 0) {
+	if (send_int(cd->fd, llist_length(avail)) < 0) {
 		err = -1;
 		goto error;
 	}
@@ -445,14 +435,14 @@ static int server_send_games(ConnData *cd, int nump)
 	/* send up to 6 games: id, name */
 	while (cur) {
 		Game *game = cur->data;
-		num = htonl(game->id);
-		if (send_msg(cd->fd, &num, 4) < 0) {
+
+		if (send_int(cd->fd, game->id) < 0) {
 			err = -1;
 			goto error;
 		}
 
 
-		if (send_msg(cd->fd, game->name, strlen(game->name) + 1) < 0) {
+		if (send_str(cd->fd, game->name) < 0) {
 			err = -1;
 			goto error;
 		}
@@ -473,7 +463,7 @@ static int server_send_player_names(ConnData *cd, Game *game)
 	int j;
 	for (j = 0; j < game->num_players; j++) {
 		const char *name = game->player_name[j] ? game->player_name[j] : "";
-		if (send_msg(cd->fd, name, strlen(name) + 1) < 0)
+		if (send_str(cd->fd, name) < 0)
 			return -1;
 
 		do_log(cd, " > sent player %d name '%s'\n", j,
@@ -484,8 +474,7 @@ static int server_send_player_names(ConnData *cd, Game *game)
 
 static int server_get_first_spot(ConnData *cd, Game *game)
 {
-	int tmp = htonl(game->first_avail_spot);
-	if (send_msg(cd->fd, &tmp, 4) < 0)
+	if (send_int(cd->fd, game->first_avail_spot) < 0)
 		return -1;
 
 	do_log(cd, " < sent %d (first available spot)\n", game->first_avail_spot);
@@ -523,14 +512,14 @@ static Game *server_do_reserve_game(ConnData *cd, int game_id)
 	game = get_game(game_id);
 	if (game == NULL) {
 		do_log(cd, "   game %d does not exist\n", game_id);
-		send_msg(cd->fd, "ERROR1", 7);
+		send_str(cd->fd, "ERROR1");
 		game = NULL;
 		goto error;
 	} else
 		do_log(cd, "   game %d is '%s'\n", game_id, game->name);
 
 
-	if (send_msg(cd->fd, "OK", 3) < 0) {
+	if (send_str(cd->fd, "OK") < 0) {
 		game = NULL;
 		goto error;
 	}
@@ -551,13 +540,11 @@ error:
 
 static int server_get_game_id(ConnData *cd)
 {
-	unsigned char len;
 	int game_id;
 
-	if (read_msg(cd->fd, &game_id, 4, &len) < 0) {
+	if (read_int(cd->fd, &game_id) < 0) {
 		return -1;
 	}
-	game_id = ntohl(game_id);
 
 	return game_id;
 }
@@ -565,16 +552,12 @@ static int server_get_game_id(ConnData *cd)
 static int server_get_new_playernum(ConnData *cd)
 {
 	int mynump;
-	unsigned char len;
 
-	if (read_msg(cd->fd, &mynump, 4, &len) < 0) {
+	if (read_int(cd->fd, &mynump) < 0) {
 		return -1;
 	}
 
-	mynump = ntohl(mynump);
-
-
-	if (send_msg(cd->fd, "OK", 3) < 0) {
+	if (send_str(cd->fd, "OK") < 0) {
 		return -1;
 	}
 
@@ -584,12 +567,10 @@ static int server_get_new_playernum(ConnData *cd)
 static char *server_get_new_playername(ConnData *cd)
 {
 	char buf[256];
-	unsigned char len;
 
-	if (read_msg(cd->fd, buf, 255, &len) < 0) {
+	if (read_str(cd->fd, buf, 255) < 0) {
 		return NULL;
 	}
-	buf[254] = '\0';
 
 	return strdup(buf);
 }	
@@ -597,13 +578,10 @@ static char *server_get_new_playername(ConnData *cd)
 static int server_wait_for_ping(ConnData *cd)
 {
 	char buf[256];
-	unsigned char len;
 
-	if (read_msg(cd->fd, buf, 255, &len) < 0)
+	if (read_str(cd->fd, buf, 255) < 0)
 		return -1;
 
-	buf[254] = '\0';
-
 	if (strcmp("READY?", buf) != 0) {
 		do_log(cd, "protocol error (got %s)\n", buf);
 		return -1;
@@ -613,8 +591,7 @@ static int server_wait_for_ping(ConnData *cd)
 
 static int server_send_waiting_for_players(ConnData *cd, int nb_missing_players)
 {
-	int tmp = htonl(nb_missing_players);
-	if (send_msg(cd->fd, &tmp, 4) < 0)
+	if (send_int(cd->fd, nb_missing_players) < 0)
 		return -1;
 
 	return 0;
@@ -623,12 +600,11 @@ static int server_send_waiting_for_players(ConnData *cd, int nb_missing_players)
 static int server_send_last_join(ConnData *cd, int player_num, const char *player_name)
 {
 	const char *name = player_name ? player_name : "";
-	int tmp = htonl(player_num);
 
-	if (send_msg(cd->fd, &tmp, 4) < 0)
+	if (send_int(cd->fd, player_num) < 0)
 		return -1;
 
-	if (send_msg(cd->fd, name, strlen(name)+1) < 0)
+	if (send_str(cd->fd, name) < 0)
 		return -1;
 	
 	return 0;
@@ -653,6 +629,9 @@ static ConnState get_state_from_command(ConnData *cd, const char *command)
 	if (cd->state == CONN_NEW && !strcmp(command, "LISTGAME"))
 		return do_transition(cd, CONN_SETUP_LISTGAME);
 	
+	if (cd->state == CONN_NEW && !strcmp(command, "TALK_MODE"))
+		return do_transition(cd, CONN_SETUP_TALK_MODE);
+	
 	/* Games listed, doing preliminary join (game info, spot reserve) */
 	if (cd->state == CONN_SETUP_LISTGAME && !strcmp(command, "PRELJOIN"))
 		return do_transition(cd, CONN_SETUP_PRELJOIN);
@@ -685,6 +664,9 @@ static ConnState get_state_from_command(ConnData *cd, const char *command)
 	if (cd->state == CONN_START_PLAY && !strcmp(command, "PLAY"))
 		return do_transition(cd, CONN_PLAY);
 
+	if (cd->state == CONN_SETUP_TALK_MODE && !strcmp(command, "WAIT"))
+		return do_transition(cd, CONN_WAIT_MESSAGE);
+	
 	return do_transition(cd, CONN_KILL);
 }
 
@@ -707,15 +689,11 @@ static ConnState get_state_from_play_command(ConnData *cd, const char command)
 
 static ConnState server_handle_conn_new(ConnData *cd)
 {
-	unsigned char len = 0;
 	char buf[256];
 
 	set_timeouts(cd, 10);
 
-	if (read_msg(cd->fd, buf, 255, &len) < 0)
-		return CONN_KILL;
-
-	if (len == 0)
+	if (read_str(cd->fd, buf, 255) < 0)
 		return CONN_KILL;
 
 	return get_state_from_command(cd, buf);
@@ -735,7 +713,6 @@ static ConnState server_handle_conn_setup_new(ConnData *cd)
 
 static ConnState server_handle_conn_setup_listgame(ConnData *cd)
 {
-	unsigned char len = 0;
 	char buf[256];
 
 	int nump = 0;
@@ -751,7 +728,7 @@ static ConnState server_handle_conn_setup_listgame(ConnData *cd)
 	if (server_send_games(cd, nump) < 0)
 		return CONN_KILL;
 
-	if (read_msg(cd->fd, buf, 255, &len) < 0)
+	if (read_str(cd->fd, buf, 255) < 0)
 		return CONN_KILL;
 
 	return get_state_from_command(cd, buf);
@@ -762,7 +739,6 @@ static ConnState server_handle_conn_setup_preljoin(ConnData *cd)
 	/* preliminary join. user clicked on game.
 	 * let him fill his name, bump first_avail_spot
 	 */
-	unsigned char len = 0;
 	char buf[256];
 	int game_id;
 	Game *game;
@@ -783,7 +759,7 @@ static ConnState server_handle_conn_setup_preljoin(ConnData *cd)
 	if (cd->previously_reserved < 0)
 		return CONN_KILL;
 
-	if (read_msg(cd->fd, buf, 255, &len) < 0)
+	if (read_str(cd->fd, buf, 255) < 0)
 		return CONN_KILL;
 
 	return get_state_from_command(cd, buf);
@@ -839,6 +815,117 @@ static ConnState server_handle_conn_setup_join(ConnData *cd)
 	return get_state_from_command(cd, "WAIT_PLAYERS");
 }
 
+static ConnState server_handle_conn_setup_talk_mode(ConnData *cd)
+{
+	/* player joins reserved game. */
+	int32_t mynump, game_id;
+
+	cd->talk_mode = TRUE;
+
+	if (read_int(cd->fd, &game_id) < 0) {
+		do_log(cd, " error: couldn't read game id.\n");
+		SDL_UnlockMutex (gamelist_mutex);
+		return CONN_KILL;
+	}
+	
+	if (read_int(cd->fd, &mynump) < 0) {
+		do_log(cd, " error: couldn't read game id.\n");
+		SDL_UnlockMutex (gamelist_mutex);
+		return CONN_KILL;
+	}
+	
+	SDL_LockMutex (gamelist_mutex);
+
+	cd->game = get_game(game_id);
+
+	if (cd->game == NULL || cd->game->killed) {
+		do_log(cd, " error: Invalid game id %d.\n", game_id);
+		SDL_UnlockMutex (gamelist_mutex);
+		return CONN_KILL;
+	}
+
+	SDL_UnlockMutex (gamelist_mutex);
+
+	cd->player_num = mynump;
+
+	send_str(cd->fd, "OK");
+
+	do_log(cd, " entered talk mode for player %d on game %d.\n",
+		mynump, game_id);
+
+	return CONN_WAIT_MESSAGE;
+}
+
+static ConnState server_handle_conn_wait_message(ConnData *cd)
+{
+	if (sock_select(cd->fd, 500) == -1) {
+		ConnState new_state;
+	
+		SDL_LockMutex(gamelist_mutex);
+
+		if (cd->game->killed)
+			new_state = CONN_KILL;
+		else if (cd->game->message_id > cd->game->players_msg_id[cd->player_num])
+			new_state = CONN_RECV_MESSAGE;
+		else
+			new_state = CONN_WAIT_MESSAGE;
+
+		SDL_UnlockMutex(gamelist_mutex);
+
+		return new_state;
+	} else
+		return CONN_SEND_MESSAGE;
+}
+
+static ConnState server_handle_conn_send_message(ConnData *cd)
+{
+	char buf[256];
+	
+	if (read_str(cd->fd, buf, 255) < 0)
+		return CONN_KILL;
+
+	SDL_LockMutex(gamelist_mutex);
+	if (cd->game->killed) {
+		SDL_UnlockMutex(gamelist_mutex);
+		return CONN_KILL;
+	}
+
+	do_log(cd, " pushing message '%s' from player %d.\n", buf, cd->player_num);
+	game_push_message(cd->game, cd->player_num, buf);
+	SDL_UnlockMutex(gamelist_mutex);
+
+	return CONN_WAIT_MESSAGE;
+}
+
+static ConnState server_handle_conn_recv_message(ConnData *cd)
+{
+	int msg_id;
+	const Message *msg = NULL;
+
+	SDL_LockMutex(gamelist_mutex);
+
+	if (cd->game->killed) {
+		SDL_UnlockMutex(gamelist_mutex);
+		return CONN_KILL;
+	}
+
+	msg_id = cd->game->players_msg_id[cd->player_num] + 1;
+
+	msg = &cd->game->messages[msg_id];
+	if (send_int(cd->fd, msg->player) < 0 ||
+	    send_str(cd->fd, msg->message) < 0) {
+		SDL_UnlockMutex(gamelist_mutex);
+		return CONN_KILL;
+	}
+	do_log(cd, " received message '%s' from player %d.\n",
+		msg->message, msg->player);
+
+	cd->game->players_msg_id[cd->player_num] = msg_id;
+	SDL_UnlockMutex(gamelist_mutex);
+
+	return CONN_WAIT_MESSAGE;
+}
+
 static ConnState server_handle_conn_wait_players(ConnData *cd)
 {
 	/* The game is now setup and we just have to wait for all players to
@@ -902,15 +989,11 @@ static ConnState server_handle_conn_start_play(ConnData *cd)
 	int i;
 
 	SDL_LockMutex(gamelist_mutex);
-	cd->game->move_id = 0;
-	cd->game->cur_x = -1;
-	cd->game->cur_y = -1;
 	cd->game->quit_players = 0;
 	cd->game->killed = 0;
 
 	for (i = 0; i < cd->game->num_players; i++) {
-		if (send_msg(cd->fd, cd->game->player_name[i], 
-				strlen(cd->game->player_name[i])+1) < 0) {
+		if (send_str(cd->fd, cd->game->player_name[i]) < 0) {
 			SDL_UnlockMutex(gamelist_mutex);
 			return CONN_KILL;
 		}
@@ -921,12 +1004,10 @@ static ConnState server_handle_conn_start_play(ConnData *cd)
 
 static ConnState server_handle_conn_play(ConnData *cd)
 {
-	unsigned char len = 0;
 	char action;
-
-	if (read_msg(cd->fd, &action, 1, &len) < 0) {
+	int r = read_char(cd->fd, &action);
+	if (r != sizeof(char))
 		return CONN_KILL;
-	}
 
 	return get_state_from_play_command(cd, action);
 }
@@ -934,52 +1015,66 @@ static ConnState server_handle_conn_play(ConnData *cd)
 static ConnState server_handle_conn_play_receive(ConnData *cd)
 {
 	int playercolor, x, y;
-	unsigned char len = 0;
 
-	if (read_msg(cd->fd, &playercolor, 4, &len) < 0)
+	if (read_int(cd->fd, &playercolor) < 0)
 		return CONN_KILL;
 
-	playercolor = ntohl(playercolor);
-
 	SDL_LockMutex(gamelist_mutex);
 
 	do_log(cd, " > game %d: player %d waiting for player %d move\n",
 		cd->game->id, cd->player_num, playercolor);
 
-
-	while (cd->game->move_id <= cd->move_id) {
+	/* As long as no move has been posted to the game since this player's move id,
+	 * wait. */
+	while (cd->game->move_id <= cd->game->players_move_id[cd->player_num]) {
+		do_log(cd, "   game %d: nothing to send to player %d, waiting (id %d/%d)\n", 
+			cd->game->id, cd->player_num, cd->game->players_move_id[cd->player_num],
+			cd->game->move_id);
 		SDL_CondWait(cd->game->cond_move, gamelist_mutex);
 		if (cd->game->killed)
 			break;
 	}
 
-	if (!cd->game->killed) {
-		x = htonl(cd->game->cur_x);
-		y = htonl(cd->game->cur_y);
-	} else {
-		x = htonl(-2);
-		y = htonl(-2);
-	}
+	while (cd->game->players_move_id[cd->player_num] < cd->game->move_id) {
+		int move_id = cd->game->players_move_id[cd->player_num] + 1;
 
-	if (send_msg(cd->fd, &x, 4) < 0) {
-		cd->game->waiting_acks--;
-		SDL_CondBroadcast(cd->game->cond_wait);
-		SDL_UnlockMutex(gamelist_mutex);
-		return CONN_KILL;
-	}
-	if (send_msg(cd->fd, &y, 4) < 0) {
-		cd->game->waiting_acks--;
-		SDL_CondBroadcast(cd->game->cond_wait);
-		SDL_UnlockMutex(gamelist_mutex);
-		return CONN_KILL;
-	}
-	do_log(cd, " < game %d: pushed move ID %d"
-		   " (x %d, y %d) from player %d to player %d\n",
-		cd->game->id, cd->game->move_id,
-		x, y, playercolor, cd->player_num);
-	cd->move_id++;
+		if (!cd->game->killed) {
+			x = cd->game->moves[move_id].x;
+			y = cd->game->moves[move_id].y;
+		} else {
+			x = -2;
+			y = -2;
+		}
 
-	cd->game->waiting_acks--;
+		SDL_UnlockMutex(gamelist_mutex);
+		if (send_int(cd->fd, x) < 0) {
+			SDL_LockMutex(gamelist_mutex);
+			SDL_CondBroadcast(cd->game->cond_wait);
+			SDL_UnlockMutex(gamelist_mutex);
+			return CONN_KILL;
+		}
+		if (send_int(cd->fd, y) < 0) {
+			SDL_LockMutex(gamelist_mutex);
+			SDL_CondBroadcast(cd->game->cond_wait);
+			SDL_UnlockMutex(gamelist_mutex);
+			return CONN_KILL;
+		}
+		SDL_LockMutex(gamelist_mutex);
+
+		if (cd->game->killed) {
+			SDL_CondBroadcast(cd->game->cond_wait);
+			SDL_UnlockMutex(gamelist_mutex);
+			return CONN_KILL;
+		}
+
+		do_log(cd, " < game %d: pushed move ID %d"
+			   " (x %d, y %d) from player %d to player %d\n",
+			cd->game->id, move_id, x, y, cd->game->moves[move_id].player,
+			cd->player_num);
+
+		cd->game->players_move_id[cd->player_num] = move_id;
+
+	}
 	SDL_CondBroadcast(cd->game->cond_wait);
 	SDL_UnlockMutex(gamelist_mutex);
 
@@ -996,13 +1091,27 @@ static void kill_game(Game *game)
 	SDL_CondBroadcast(game->cond_move);
 }
 
+static int players_lagging(ConnData *cd)
+{
+	int i;
+
+	for (i = 0; i < cd->game->num_players; i++) {
+		if (cd->game->players_move_id[i] < cd->game->move_id) {
+			do_log(cd, "   game %d lagging: player %d is at move %d, while "
+			       "game is at %d\n", cd->game->id, i, 
+				cd->game->players_move_id[i], cd->game->move_id);
+			return 1;
+		}
+	}
+	return 0;
+}
+
 static ConnState server_handle_conn_play_send(ConnData *cd, int quit)
 {
 	int playercolor, x, y;
-	unsigned char len = 0;
 
 	SDL_LockMutex(gamelist_mutex);
-	while (cd->game->waiting_acks > 0) {
+	while (players_lagging(cd)) {
 		/* wait for the previous move to be received by 
 		 * all */
 		SDL_CondWait(cd->game->cond_wait, gamelist_mutex);
@@ -1012,29 +1121,22 @@ static ConnState server_handle_conn_play_send(ConnData *cd, int quit)
 		}
 	}
 
-	if (read_msg(cd->fd, &playercolor, 4, &len) < 0) {
+	if (read_int(cd->fd, &playercolor) < 0) {
 		SDL_UnlockMutex(gamelist_mutex);
 		return CONN_KILL;
 	}
 
-	playercolor = ntohl(playercolor);
 	if (playercolor != cd->player_num) {
 		SDL_UnlockMutex(gamelist_mutex);
 		return CONN_KILL;
 	}
 
 	if (!quit) {
-		if (read_msg(cd->fd, &x, 4, &len) < 0) {
+		if (read_int(cd->fd, &x) < 0
+		 || read_int(cd->fd, &y) < 0) {
 			SDL_UnlockMutex(gamelist_mutex);
 			return CONN_KILL;
 		}
-		if (read_msg(cd->fd, &y, 4, &len) < 0) {
-			SDL_UnlockMutex(gamelist_mutex);
-			return CONN_KILL;
-		}
-
-		x = ntohl(x);
-		y = ntohl(y);
 	} else {
 		x = -2;
 		y = -2;
@@ -1045,16 +1147,12 @@ static ConnState server_handle_conn_play_send(ConnData *cd, int quit)
 		return CONN_KILL;
 	}
 
-	cd->game->cur_player = playercolor;
-	cd->game->cur_x = x;
-	cd->game->cur_y = y;
-	cd->game->move_id++;
-	cd->game->waiting_acks = cd->game->num_remote;
-
-	SDL_CondBroadcast(cd->game->cond_move);
-
-	do_log(cd, " < game %d: player %d sends move ID %d (x %d, y %d)\n",
-		cd->game->id, playercolor, cd->game->move_id, x, y);
+	if (x != -1 && y != -1) {
+		game_push_move(cd->game, playercolor, x, y);
+	
+		do_log(cd, " < game %d: player %d sends move ID %d (x %d, y %d)\n",
+			cd->game->id, playercolor, cd->game->move_id, x, y);
+	}
 
 	if (quit) {
 		SDL_UnlockMutex(gamelist_mutex);
@@ -1063,7 +1161,7 @@ static ConnState server_handle_conn_play_send(ConnData *cd, int quit)
 
 	/* wait here that everyone gets the move. 
 	 * this avoids deadlocks */
-	while (cd->game->waiting_acks > 0) {
+	while (players_lagging(cd)) {
 		SDL_CondWait(cd->game->cond_wait, gamelist_mutex);
 		if (cd->game->killed) {
 			SDL_UnlockMutex(gamelist_mutex);
@@ -1079,6 +1177,14 @@ static ConnState server_handle_conn_kill(ConnData *cd)
 {
 	do_log(cd, "Connection done\n", cd);
 
+	if (cd->talk_mode)
+	{
+		do_log(cd, "Shutting down talk connection\n");
+		sock_close(cd->fd);
+
+		return CONN_EXIT;
+	}
+
 	SDL_LockMutex(gamelist_mutex);
 
 	if (cd->previously_reserved > 0)
@@ -1111,8 +1217,8 @@ static ConnState server_handle_conn_kill(ConnData *cd)
 
 		if (cd->am_initiator)
 		{
-			/* Only the initiator finalizes the game kill, for
-			 * obvious double-free reasons */
+			/* Only the initiator (and not its talk socket) finalizes
+			 * the game kill, for obvious double-free reasons */
 			do_log(cd, "waiting for %d quit acks before finalizing "
 				"game %d kill\n",
 				cd->game->num_players 
@@ -1128,6 +1234,10 @@ static ConnState server_handle_conn_kill(ConnData *cd)
 			do_log(cd, "finalizing game %d kill\n", cd->game->id);
 
 			gamelist = llist_remove(gamelist, cd->game);
+			
+			if (dump_dir && cd->game->move_id > 1)
+				dump_game(cd->game);
+
 			free_game(cd->game);
 			cd->game = NULL;
 		}
@@ -1154,6 +1264,8 @@ static ConnState server_handle_state(ConnData *cd)
 		return server_handle_conn_setup_preljoin(cd);
 	case CONN_SETUP_JOIN:
 		return server_handle_conn_setup_join(cd);
+	case CONN_SETUP_TALK_MODE:
+		return server_handle_conn_setup_talk_mode(cd);
 	case CONN_WAIT_PLAYERS:
 		return server_handle_conn_wait_players(cd);
 	case CONN_START_PLAY:
@@ -1168,6 +1280,12 @@ static ConnState server_handle_state(ConnData *cd)
 		return server_handle_conn_play_send(cd, 1);
 	case CONN_KILL:
 		return server_handle_conn_kill(cd);
+	case CONN_WAIT_MESSAGE:
+		return server_handle_conn_wait_message(cd);
+	case CONN_SEND_MESSAGE:
+		return server_handle_conn_send_message(cd);
+	case CONN_RECV_MESSAGE:
+		return server_handle_conn_recv_message(cd);
 	default:
 		return CONN_EXIT;
 	}
@@ -1281,8 +1399,9 @@ static void broadcast_once(void)
 		broadcast_setup(broadcastsock);
 	}
 	if (broadcastsock < 0) {
+                int err = get_net_error();
 		do_log(NULL, "Can't broadcast: %s (%d)\n",
-			strerror(errno), errno);
+			net_err_str(err), err);
 		return;
 	}
 
@@ -1296,17 +1415,13 @@ static void broadcast_once(void)
 		0, (struct sockaddr *)&br_addr,
 		sizeof(br_addr));
 	if (err < 0) {
-#ifdef __MINGW32__
-		err = WSAGetLastError();
-#else
-		err = errno;
-#endif
+		err = get_net_error();
 		do_log(NULL, "Can't send broadcast: %s (%d)\n",
-			strerror(err), err);
+			net_err_str(err), err);
 	}
 }
 
-int server_start(const char *logfile)
+int server_start(const char *logfile, const char *game_dump_dir)
 {
 	int clifd;
 #ifndef __MINGW32__
@@ -1325,10 +1440,13 @@ int server_start(const char *logfile)
 		}
 	} else
 		logfp = stdout;
-	
+
+	if (game_dump_dir)
+		dump_dir = game_dump_dir;
+
 	serversock = socket(AF_INET, SOCK_STREAM, 0);
 	if (serversock < 0) {
-        	perror("Error on opening socket");
+		perror("Error on opening socket");
 		return -1;
 	}
 	
