@@ -28,8 +28,8 @@
 #ifndef __SERVER_H__
 #define __SERVER_H__
 
-int server_start(const char *logfile);
-int server_stop(void);
+int server_start(const char *logfile, const char *game_dump_dir);
+void server_stop(void);
 void server_set_select_interval(int timeout);
 void server_set_do_broadcast(int enable, int magic);
 
