@@ -88,7 +88,7 @@ static TileType tile_get_type(int x, int y)
 static SDL_Surface *tiles_models[NUM_TILE_TYPES] = {NULL};
 static LList *my_tiles = NULL;
 
-void init_tiles_models(void)
+static void init_tiles_models(void)
 {
 	int i = 0;
 	assert(tiles_models[TILE_NO_TILE] == NULL);
@@ -192,7 +192,7 @@ void tile_draw (Tile *tile)
 	tile_draw_fast(tile, !is_playing());
 }
 
-int tile_is_forward(Tile *tile, Tile *third)
+static int tile_is_forward(Tile *tile, Tile *third)
 {
 	if (!tile->pawn || third->pawn)
 		return FALSE;
