@@ -23,6 +23,7 @@
 #include <SDL.h>
 #include <assert.h>
 #include "llist.h"
+#include "font.h"
 
 //#define DEBUG 1
 
@@ -55,8 +56,14 @@
 #define PAWN_OFFSET 10
 #define ARROW_OFFSET_FROM_CENTER 15
 #define G_MSG_X 65
-#define G_MSG_Y (YS-25)
+#define G_MSG_Y (YS - 25)
 
+#define G_TALK_X 65
+#define G_TALK_Y (YS - 45)
+#define MAX_MSG_LEN (((XS -  G_TALK_X) / CHARWIDTH) - 2)
+
+#define HELP_X 10
+#define HELP_Y 10
 #define MAX_TILES_X 9
 #define MAX_TILES_Y 9
 
@@ -75,6 +82,7 @@
 #define PLAYER_NAME_LEN	32
 #define GAME_NAME_LEN 	32
 #define SERVER_LEN 	64
+#define SAVED_LEN	256
 
 extern char *progpath;
 extern char *langpath;
@@ -115,6 +123,9 @@ int event_poll(void);
 int handle_generic_event(SDL_Event *event);
 int delay_with_event_poll(int msec_timeout);
 
+const char *get_desktop_folder(void);
+const char *get_home_folder(void);
+int folder_exists(const char *folder);
 #ifdef MAEMO
 void SetWMName(const char *name, int is_fullscreen);
 #endif
